package org.jivesoftware.openfire.plugin;

import org.jivesoftware.openfire.user.UserManager;
import org.jivesoftware.openfire.user.UserNotFoundException;
import org.jivesoftware.openfire.user.User;
import org.jivesoftware.util.Log;
import java.util.Calendar;
import org.jivesoftware.database.DbConnectionManager;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import org.jivesoftware.util.JiveGlobals;

import org.jivesoftware.util.EmailService;
import org.jivesoftware.util.StringUtils;

public class CheckParam{
	private static final String USERNAME_TOKEN
						= "SELECT count(*) FROM jivepassservice " +
								"where username = ? " +
								  "and token = ? " +
								  "and createTime >= ?";
	private UserManager uMgr;
	private User u;
	
	
	public CheckParam(){
		uMgr = UserManager.getInstance();
	}
	// check that the given username is valid and exists!
	// and initialise the User Object with the given username!
	public boolean checkUsername(String username){
		try{
			u = uMgr.getUser(username);
			return true;
		}catch(UserNotFoundException unfe){
			return false;
		}
	}
	// check that the given username have the specified email address
	public boolean checkUsernameToEmail(String username, String email){
		if(checkUsername(username)){
			if(u.getEmail().equalsIgnoreCase(email)){
				return true;
			}
		}
		return false;
	}
	
	// reset the password for the given username and send a email to the user!
	// the sender could also be specified!
	public boolean resetPass(String username, String fromMail){
		if(checkUsername(username)){
			String str = StringUtils.randomString(8);
			u.setPassword(str);
			EmailService email = EmailService.getInstance();
			email.sendMessage(""
							, u.getEmail()
							, ""
							, fromMail
							, "New Password for Spark Account"
							, null
							, "Your new Password for your spark account is:" +
							  "<BR>Username: " + u.getUsername() +
							  "<BR>Password: " + str
							);
			return true;
		}
		return false;
	}
	
	// check that the given username and the token are valid
	// and that the token isn't expired!
	public boolean checkUsernameAndToken(String username, String token){
		
		boolean bUserFound = false;
		Connection con = null;
		ResultSet rs = null;
		PreparedStatement prepStmt = null;
		long lTime = 0;
		
		try{
			con = DbConnectionManager.getConnection();
			prepStmt = con.prepareStatement(USERNAME_TOKEN);
			prepStmt.setString(1, username);
			prepStmt.setString(2, token);
			lTime = Calendar.getInstance().getTimeInMillis() - (JiveGlobals.getIntProperty("plugin.passservice.lifetime", 1) * 86400000);
			prepStmt.setLong(3, lTime);
			rs = prepStmt.executeQuery();
			if( rs.next() ){
				bUserFound = ( rs.getInt(1) == 1) ? true : false;
			}
		}catch(SQLException sqlEx){
			Log.warn("Passservice:"+sqlEx);
		}finally{
			DbConnectionManager.closeResultSet(rs);
			DbConnectionManager.closeStatement(prepStmt);
			DbConnectionManager.closeConnection(con);
		}
		if(bUserFound)
		return true;
		return false;
	}
}