/**
 * $Revision: 1.0 $
 *
 * This software is published under the terms of the GNU Public License (GPL),
 * a copy of which is included in this distribution.
 * 
 * Inspired by the user service plugin from Justin Hunt.
 */

package org.jivesoftware.openfire.plugin;

import org.jivesoftware.openfire.container.Plugin;
import org.jivesoftware.openfire.container.PluginManager;
import org.jivesoftware.openfire.user.UserManager;
import org.jivesoftware.openfire.user.UserNotFoundException;
import org.jivesoftware.openfire.user.UserAlreadyExistsException;
import org.jivesoftware.openfire.user.User;
import org.jivesoftware.openfire.XMPPServer;
import org.jivesoftware.util.JiveGlobals;
import org.jivesoftware.util.StringUtils;
import org.xmpp.packet.JID;

import java.io.File;

public class PassServicePlugin implements Plugin {

    private String hostname;
    private boolean enabled;
    private String sourceEmail;
    private int lifetime;
    private boolean useSSL;

    public void initializePlugin(PluginManager manager, File pluginDirectory) {
        
        // See if the service is enabled or not.
        // initialise the values!
    	
        enabled = JiveGlobals.getBooleanProperty("plugin.passservice.enabled", false);
        sourceEmail = JiveGlobals.getProperty("plugin.passservice.sourceEmail","");
        lifetime = JiveGlobals.getIntProperty("plugin.passservice.lifetime", 1);
        useSSL = JiveGlobals.getBooleanProperty("plugin.passservice.useSSL", false);
    }

    public void destroyPlugin() {
    }
    
    public String getHostname(){
    	hostname = JiveGlobals.getProperty("plugin.passservice.hostname","");
    	return hostname;
    }
    
    public void setHostname(String hostname){
    	this.hostname = hostname;
    	JiveGlobals.setProperty("plugin.passservice.hostname", hostname);
    }
    
    public int getAdminPort(){
    	return JiveGlobals.getXMLProperty("adminConsole.port", 9090);
    }
    public int getSecureAdminPort(){
    	return JiveGlobals.getXMLProperty("adminConsole.securePort", 9091);
    }
	
    public String getSourceEmail() {
    	sourceEmail = JiveGlobals.getProperty("plugin.passservice.sourceEmail","");
    	return sourceEmail;
    }
    
    public void setSourceEmail(String sMail){
    	sourceEmail = sMail;
    	JiveGlobals.setProperty("plugin.passservice.sourceEmail", sourceEmail);
    }
    
    public int getLifetime() {
        lifetime = JiveGlobals.getIntProperty("plugin.passservice.lifetime", 1);
    	return lifetime;
    }
    
    public void setLifetime(int life){
    	lifetime = life;
    	JiveGlobals.setProperty("plugin.passservice.lifetime", Integer.toString(lifetime) );
    }

    public boolean isEnabled() {
        enabled = JiveGlobals.getBooleanProperty("plugin.passservice.enabled", false);
        return enabled;
    }

    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
        JiveGlobals.setProperty("plugin.passservice.enabled",  enabled ? "true" : "false");
    }
    
    public boolean isSSL() {
        useSSL = JiveGlobals.getBooleanProperty("plugin.passservice.useSSL", false);
        return useSSL;
    }

    public void setSSL(boolean useSSL) {
        this.useSSL = useSSL;
        JiveGlobals.setProperty("plugin.passservice.useSSL",  useSSL ? "true" : "false");
    }
    public boolean isEmailConfigured(){
    	return JiveGlobals.getBooleanProperty("mail.configured");
    }
}