package org.jivesoftware.openfire.plugin;

import org.jivesoftware.database.DbConnectionManager;
import org.jivesoftware.util.EmailService;

import org.jivesoftware.util.Log;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Date;
import java.util.Calendar;
import java.security.SecureRandom;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Formatter;
import org.jivesoftware.util.JiveGlobals;
import org.jivesoftware.openfire.XMPPServer;

public class Token {
	
	private static final String INSERT_TOKEN = 
		"INSERT INTO jivepassservice (token,username,createTime) values (?,?,?)";
	private static final String DELETE_TOKEN =
		"DELETE FROM jivepassservice WHERE token=?";
	private static final String CLEAN_TOKENS =
		"DELETE FROM jivepassservice " +
				"where createTime <= ? ";
	
	private MessageDigest md5Prov;
	private boolean tokenGenerated;
	private String sFQDN;
	private String fMail;
	
	public Token(boolean useSSL, String fromMail, String hostname, int nPort, int secPort) throws NoSuchAlgorithmException{
		md5Prov = MessageDigest.getInstance("MD5");
		tokenGenerated = false;
		if(useSSL){
			sFQDN = "https://"+hostname+":"+secPort+"/";
		}else{
			sFQDN = "http://"+hostname+":"+nPort+"/";
		}
		this.fMail = fromMail;
	}
	
	public Token(String fromMail, String hostname, int nPort) throws NoSuchAlgorithmException{
		md5Prov = MessageDigest.getInstance("MD5");
		tokenGenerated = false;
		sFQDN = "http://"+hostname+":"+nPort+"/";
		this.fMail = fromMail;
	}
	
	// set the source token to generate the final md5 token!
	private String genSrcToken(String username, Long timeStmp){
		SecureRandom secRand = new SecureRandom();
		Long randSeed = secRand.nextLong();
		return username + timeStmp.toString() + randSeed.toString();
	}

	// generate a new token for the specified username
	// and ensure that the token won't be longer then the given size!
	public String genToken(String username, Long timeStmp, int length){
		Formatter format = new Formatter();
		String token = genSrcToken(username, timeStmp);
		md5Prov.reset();
		md5Prov.update(token.getBytes());
		for(byte b : md5Prov.digest()){
			format.format("%02x",b);
		}
		token = format.toString();
		if( token.length() > length ){
			token = token.substring(1, length);
		}
		md5Prov.reset();
		return token;
	}
	
	// insert the generated token into database!
	public void DBinsertToken(String token, String username, Long timeStmp) throws SQLException{
		Connection con = null;
		PreparedStatement prepStmt = null;
			try{
				con = DbConnectionManager.getConnection();
				prepStmt = con.prepareStatement(INSERT_TOKEN);
				prepStmt.setString(1, token);
				prepStmt.setString(2, username);
				prepStmt.setLong(3, timeStmp);
				prepStmt.execute();
				
			}catch(SQLException sqlEx){
				throw sqlEx;
			}finally{
				DbConnectionManager.closeStatement(prepStmt);
				DbConnectionManager.closeConnection(con);
			}
	}
	
	// Clean the database from expired tokens!
	public void cleanTokenInDB() throws SQLException{
		Connection con = null;
		PreparedStatement prepStmt = null;
		long cleanTimeStmp = 0;
		try{
			con = DbConnectionManager.getConnection();
			prepStmt = con.prepareStatement(CLEAN_TOKENS);
			cleanTimeStmp = Calendar.getInstance().getTimeInMillis() - ( JiveGlobals.getIntProperty("plugin.passservice.lifetime", 1) * 86400000);
			prepStmt.setLong(1, cleanTimeStmp );
			prepStmt.execute();
		}catch(SQLException sqlEx){
			throw sqlEx;
		}finally{
			DbConnectionManager.closeStatement(prepStmt);
			DbConnectionManager.closeConnection(con);
		}
	}
	
	// send the generated token via email
	public void EmailSendToken(String toMail, String username, String token){
		EmailService email = EmailService.getInstance();
		email.sendMessage(
				 "" , toMail
				,"" , fMail
				, "Confirm pass reset for spark account:  " + username
				, null
				, "Please use the following link to reset the password of your spark account.<BR>" +
				  "<A HREF=\""+sFQDN+"plugins/passservice/passservice?token="+token+"&username="+username+"&type=new\">Link to reset the password!</A>"
				);
	}
	
	// remove the specified token from DB
	public boolean DBremoveToken(String token) throws SQLException{
		Connection con = null;
		PreparedStatement prepStmt = null;
		int affectedRows = 0;
		try{
			con = DbConnectionManager.getConnection();
			prepStmt = con.prepareStatement(DELETE_TOKEN);
			prepStmt.setString(1, token);
			prepStmt.execute();
			affectedRows = prepStmt.getUpdateCount();
		}catch(SQLException sqlEx){
			throw sqlEx;
		}finally{
			DbConnectionManager.closeStatement(prepStmt);
			DbConnectionManager.closeConnection(con);
		}
		
		if(affectedRows > 0){
			return true;
		}else{
			return false;
		}
	}
	
	// reset the object!
	// to ensure if the genToken function is called again that a new token will be generated!
	// also the md5 provider must be resetted!
	public void SendAndStoreToken(String toMail, String username, int length) throws SQLException{
		Calendar cal = Calendar.getInstance();
		cleanTokenInDB();
		String token = genToken(username, cal.getTimeInMillis(), length);
		DBinsertToken(token, username, cal.getTimeInMillis());
		EmailSendToken(toMail,username,token);
	}
}
