package org.jivesoftware.openfire.plugin.passservice;

import org.jivesoftware.openfire.plugin.CheckParam;
import org.jivesoftware.openfire.plugin.Token;
import org.jivesoftware.openfire.plugin.HTMLOutput;
import org.jivesoftware.openfire.plugin.PassServicePlugin;

import org.jivesoftware.openfire.container.PluginManager;
import org.jivesoftware.openfire.XMPPServer;
import org.jivesoftware.openfire.user.UserNotFoundException;
import org.jivesoftware.openfire.user.UserAlreadyExistsException;
import org.jivesoftware.util.Log;
import org.jivesoftware.admin.AuthCheckFilter;
import org.jivesoftware.openfire.user.UserManager;
import org.jivesoftware.openfire.user.User;
import org.jivesoftware.util.JiveGlobals;
import org.jivesoftware.openfire.ServerPort;

import java.io.File;
import java.security.NoSuchAlgorithmException;

import java.sql.SQLException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;



/**
 * Servlet that makes it possible for non admin users to reset their own password if it has been lost
 * by entering the username and the email address. The servlet will look into the database and 
 * generate a token. If the user with the email address exist, the token will be send to the user
 * and the program will reset the password.
 *
 * @author Dennis Drr
 */
public class PassServiceServlet extends HttpServlet {
	
    private PassServicePlugin psPlugin;
    private PluginManager plugManager;
    private File fPlugin;


    public void init(ServletConfig servletConfig) throws ServletException {
        super.init(servletConfig);
        
        plugManager = XMPPServer.getInstance().getPluginManager();
		psPlugin = (PassServicePlugin) plugManager.getPlugin("passservice");
		fPlugin = plugManager.getPluginDirectory(psPlugin);
        // Exclude this servlet from requiring the user to login
		// Be aware that cases won't be ignored!
		// That means that passservice/passservice won't work!
        AuthCheckFilter.addExclude("passservice/passservice");
    }

    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException
    {
    	// Collect some data!
		String username = request.getParameter("username");
		String token	= request.getParameter("token");
		String email	= request.getParameter("email");
		String type		= request.getParameter("type");
		boolean useSSL	= JiveGlobals.getBooleanProperty("plugin.passservice.useSSL");
		String srcEmail = null;
		Token tok 		= null;
		
		try{
			srcEmail = getSrcEmail();
		}catch(UserNotFoundException unfe){
			throw new ServletException("The source email could not found!");
		}
	    CheckParam cpCheck = new CheckParam();		
        // Printwriter for writing out responses to browser
        PrintWriter out = response.getWriter();
        
        try{
            tok = new Token(useSSL, srcEmail, getHostname(), psPlugin.getAdminPort(), psPlugin.getSecureAdminPort());
        }catch(NoSuchAlgorithmException nsae){
        	Log.warn("Could not initialise the token service!");
        	throw new ServletException("Tokenizer could not be initialiozed!");
        }
        
		ServletContext srv = getServletContext();
		
        // Check that our plugin is enabled.
        if (!psPlugin.isEnabled()) {
            Log.warn("Passservice plugin is disabled: " + request.getQueryString());
            out.println(HTMLOutput.getErrorMessage(fPlugin, "Configuration Error","The Passservice has been disabled!"));
            return;
        }
        
        // Check that the email function is configured properly!
        // This is needed to ensure that we are able to send emails!
        if (!JiveGlobals.getBooleanProperty("mail.configured")){
        	Log.warn("You haven't configured the email function!");
        	out.println(HTMLOutput.getErrorMessage(fPlugin, "Configuration Error","The Emailservice is not configured properly!"));
        	return;
        }


        // the type new indicates that the page has been called  with username and token!
        // Then the CheckParam class checks the given parameters!
		if("new".equals(type)){
			// Check for correct username and token
			// also the lifetime will be checked!
			if( cpCheck.checkUsernameAndToken(username,token) ){
				// if everything is ok, the token will be removed from database!
				try{
					if(tok.DBremoveToken(token)){
						// and the password will be changed for the given user!
						// then the user will be informed by email!
						if(cpCheck.resetPass(username,srcEmail)){
							out.println(HTMLOutput.getHTMLFile(fPlugin, "passReset.html"));
						}else{
							out.println(HTMLOutput.getErrorMessage(fPlugin, "Error while trying to reset Password","Error: It was not possible to reset the password!"));
						}					
					}else{
					out.println(HTMLOutput.getErrorMessage(fPlugin, "Error while trying to reset Password","Error: The token could not removed from database!"));
					}
				}catch(SQLException sqlEx){
					Log.warn(sqlEx);
					out.println(HTMLOutput.getErrorMessage(fPlugin, "Connection Error","Could not connect to database!"));
				}
			}else{
				out.println(HTMLOutput.getErrorMessage(fPlugin, "Error while trying to reset Password","The given username or token is wrong! Be aware that the token has a bounded lifetime!"));
			}
		// the status type indicates that the page has been called with username and email!
		}else if("status".equals(type)){
				if ( cpCheck.checkUsernameToEmail(username,email) ) {
					try{
						tok.SendAndStoreToken(email,username,128);
						out.println(HTMLOutput.getHTMLFile(fPlugin, "confEmail.html"));
					}catch(SQLException sqlEx){
						out.println(HTMLOutput.getErrorMessage(fPlugin ,"Program Error","The token generation failed!"));
					}
				}else{
					out.println(HTMLOutput.getErrorMessage(fPlugin, "Email or Account not valid!","The given email address is not valid for the specified account!"));
				}
		// if no type has been specified we send the formular for password reset!
		}else{
			out.println(HTMLOutput.getHTMLFile(fPlugin, "formEmail.html"));
		}
		out.flush();
		out.close();
    }
    
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        doGet(request, response);
    }

    public void destroy() {
        super.destroy();
        // Release the excluded URL
        AuthCheckFilter.removeExclude("passservice/passservice");
        // remove the objects that are not needed anymore!
        // to enable a clean shutdown!
        psPlugin = null;
        plugManager = null;
        fPlugin = null;
    }

	private String getSrcEmail() throws UserNotFoundException{
        // Enable the possibility to change the from-emailaddress!
        // If plugin.passservice.sourceEmail is configured it will be used, otherwise 
        // the email address from the admin will used.
        if(JiveGlobals.getProperty("plugin.passservice.sourceEmail", "").equalsIgnoreCase("")){
        	return UserManager.getInstance().getUser("admin").getEmail();
        }else{
        	return JiveGlobals.getProperty("plugin.passservice.sourceEmail", "");
        }
	}
	private String getHostname(){
		if(JiveGlobals.getProperty("plugin.passservice.hostname", "").equalsIgnoreCase("")){
			return XMPPServer.getInstance().getServerInfo().getName();
		}else{
			return JiveGlobals.getProperty("plugin.passservice.hostname", "");
		}
	}
}